
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#include "config.h"
#include "ide-vcs-enums.h"
#include "ide-vcs-config.h"
#include "ide-vcs-file-info.h"

#define C_ENUM(v) ((gint) v)
#define C_FLAGS(v) ((guint) v)

/* enumerations from "ide-vcs-config.h" */

GType
ide_vcs_config_type_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (IDE_VCS_CONFIG_FULL_NAME), "IDE_VCS_CONFIG_FULL_NAME", "full-name" },
        { C_ENUM (IDE_VCS_CONFIG_EMAIL), "IDE_VCS_CONFIG_EMAIL", "email" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("IdeVcsConfigType"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "ide-vcs-file-info.h" */

GType
ide_vcs_file_status_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (IDE_VCS_FILE_STATUS_IGNORED), "IDE_VCS_FILE_STATUS_IGNORED", "ignored" },
        { C_ENUM (IDE_VCS_FILE_STATUS_UNCHANGED), "IDE_VCS_FILE_STATUS_UNCHANGED", "unchanged" },
        { C_ENUM (IDE_VCS_FILE_STATUS_UNTRACKED), "IDE_VCS_FILE_STATUS_UNTRACKED", "untracked" },
        { C_ENUM (IDE_VCS_FILE_STATUS_ADDED), "IDE_VCS_FILE_STATUS_ADDED", "added" },
        { C_ENUM (IDE_VCS_FILE_STATUS_RENAMED), "IDE_VCS_FILE_STATUS_RENAMED", "renamed" },
        { C_ENUM (IDE_VCS_FILE_STATUS_DELETED), "IDE_VCS_FILE_STATUS_DELETED", "deleted" },
        { C_ENUM (IDE_VCS_FILE_STATUS_CHANGED), "IDE_VCS_FILE_STATUS_CHANGED", "changed" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("IdeVcsFileStatus"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* Generated data ends here */

