
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#include "config.h"
#include "ide-tree-enums.h"
#include "ide-tree-node.h"

#define C_ENUM(v) ((gint) v)
#define C_FLAGS(v) ((guint) v)

/* enumerations from "ide-tree-node.h" */

GType
ide_tree_node_visit_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (IDE_TREE_NODE_VISIT_BREAK), "IDE_TREE_NODE_VISIT_BREAK", "break" },
        { C_ENUM (IDE_TREE_NODE_VISIT_CONTINUE), "IDE_TREE_NODE_VISIT_CONTINUE", "continue" },
        { C_ENUM (IDE_TREE_NODE_VISIT_CHILDREN), "IDE_TREE_NODE_VISIT_CHILDREN", "children" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("IdeTreeNodeVisit"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
ide_tree_node_flags_get_type (void)
{
    static gsize gtype_id = 0;
    static const GFlagsValue values[] = {
        { C_FLAGS (IDE_TREE_NODE_FLAGS_NONE), "IDE_TREE_NODE_FLAGS_NONE", "none" },
        { C_FLAGS (IDE_TREE_NODE_FLAGS_DESCENDANT), "IDE_TREE_NODE_FLAGS_DESCENDANT", "descendant" },
        { C_FLAGS (IDE_TREE_NODE_FLAGS_ADDED), "IDE_TREE_NODE_FLAGS_ADDED", "added" },
        { C_FLAGS (IDE_TREE_NODE_FLAGS_CHANGED), "IDE_TREE_NODE_FLAGS_CHANGED", "changed" },
        { C_FLAGS (IDE_TREE_NODE_FLAGS_REMOVED), "IDE_TREE_NODE_FLAGS_REMOVED", "removed" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_flags_register_static (g_intern_static_string ("IdeTreeNodeFlags"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* Generated data ends here */

