/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __IPC_GIT_CONFIG_H__
#define __IPC_GIT_CONFIG_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.gnome.Builder.Git.Config */

#define IPC_TYPE_GIT_CONFIG (ipc_git_config_get_type ())
#define IPC_GIT_CONFIG(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), IPC_TYPE_GIT_CONFIG, IpcGitConfig))
#define IPC_IS_GIT_CONFIG(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), IPC_TYPE_GIT_CONFIG))
#define IPC_GIT_CONFIG_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), IPC_TYPE_GIT_CONFIG, IpcGitConfigIface))

struct _IpcGitConfig;
typedef struct _IpcGitConfig IpcGitConfig;
typedef struct _IpcGitConfigIface IpcGitConfigIface;

struct _IpcGitConfigIface
{
  GTypeInterface parent_iface;


  gboolean (*handle_close) (
    IpcGitConfig *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_read_key) (
    IpcGitConfig *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_key);

  gboolean (*handle_write_key) (
    IpcGitConfig *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_key,
    const gchar *arg_value);

  void (*closed) (
    IpcGitConfig *object);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (IpcGitConfig, g_object_unref)
#endif

GType ipc_git_config_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *ipc_git_config_interface_info (void);
guint ipc_git_config_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void ipc_git_config_complete_read_key (
    IpcGitConfig *object,
    GDBusMethodInvocation *invocation,
    const gchar *value);

void ipc_git_config_complete_write_key (
    IpcGitConfig *object,
    GDBusMethodInvocation *invocation);

void ipc_git_config_complete_close (
    IpcGitConfig *object,
    GDBusMethodInvocation *invocation);



/* D-Bus signal emissions functions: */
void ipc_git_config_emit_closed (
    IpcGitConfig *object);



/* D-Bus method calls: */
void ipc_git_config_call_read_key (
    IpcGitConfig *proxy,
    const gchar *arg_key,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean ipc_git_config_call_read_key_finish (
    IpcGitConfig *proxy,
    gchar ** out_value,
    GAsyncResult *res,
    GError **error);

gboolean ipc_git_config_call_read_key_sync (
    IpcGitConfig *proxy,
    const gchar *arg_key,
    gchar ** out_value,
    GCancellable *cancellable,
    GError **error);

void ipc_git_config_call_write_key (
    IpcGitConfig *proxy,
    const gchar *arg_key,
    const gchar *arg_value,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean ipc_git_config_call_write_key_finish (
    IpcGitConfig *proxy,
    GAsyncResult *res,
    GError **error);

gboolean ipc_git_config_call_write_key_sync (
    IpcGitConfig *proxy,
    const gchar *arg_key,
    const gchar *arg_value,
    GCancellable *cancellable,
    GError **error);

void ipc_git_config_call_close (
    IpcGitConfig *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean ipc_git_config_call_close_finish (
    IpcGitConfig *proxy,
    GAsyncResult *res,
    GError **error);

gboolean ipc_git_config_call_close_sync (
    IpcGitConfig *proxy,
    GCancellable *cancellable,
    GError **error);



/* ---- */

#define IPC_TYPE_GIT_CONFIG_PROXY (ipc_git_config_proxy_get_type ())
#define IPC_GIT_CONFIG_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), IPC_TYPE_GIT_CONFIG_PROXY, IpcGitConfigProxy))
#define IPC_GIT_CONFIG_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), IPC_TYPE_GIT_CONFIG_PROXY, IpcGitConfigProxyClass))
#define IPC_GIT_CONFIG_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), IPC_TYPE_GIT_CONFIG_PROXY, IpcGitConfigProxyClass))
#define IPC_IS_GIT_CONFIG_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), IPC_TYPE_GIT_CONFIG_PROXY))
#define IPC_IS_GIT_CONFIG_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), IPC_TYPE_GIT_CONFIG_PROXY))

typedef struct _IpcGitConfigProxy IpcGitConfigProxy;
typedef struct _IpcGitConfigProxyClass IpcGitConfigProxyClass;
typedef struct _IpcGitConfigProxyPrivate IpcGitConfigProxyPrivate;

struct _IpcGitConfigProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  IpcGitConfigProxyPrivate *priv;
};

struct _IpcGitConfigProxyClass
{
  GDBusProxyClass parent_class;
};

GType ipc_git_config_proxy_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (IpcGitConfigProxy, g_object_unref)
#endif

void ipc_git_config_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
IpcGitConfig *ipc_git_config_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
IpcGitConfig *ipc_git_config_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void ipc_git_config_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
IpcGitConfig *ipc_git_config_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
IpcGitConfig *ipc_git_config_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define IPC_TYPE_GIT_CONFIG_SKELETON (ipc_git_config_skeleton_get_type ())
#define IPC_GIT_CONFIG_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), IPC_TYPE_GIT_CONFIG_SKELETON, IpcGitConfigSkeleton))
#define IPC_GIT_CONFIG_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), IPC_TYPE_GIT_CONFIG_SKELETON, IpcGitConfigSkeletonClass))
#define IPC_GIT_CONFIG_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), IPC_TYPE_GIT_CONFIG_SKELETON, IpcGitConfigSkeletonClass))
#define IPC_IS_GIT_CONFIG_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), IPC_TYPE_GIT_CONFIG_SKELETON))
#define IPC_IS_GIT_CONFIG_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), IPC_TYPE_GIT_CONFIG_SKELETON))

typedef struct _IpcGitConfigSkeleton IpcGitConfigSkeleton;
typedef struct _IpcGitConfigSkeletonClass IpcGitConfigSkeletonClass;
typedef struct _IpcGitConfigSkeletonPrivate IpcGitConfigSkeletonPrivate;

struct _IpcGitConfigSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  IpcGitConfigSkeletonPrivate *priv;
};

struct _IpcGitConfigSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType ipc_git_config_skeleton_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (IpcGitConfigSkeleton, g_object_unref)
#endif

IpcGitConfig *ipc_git_config_skeleton_new (void);


G_END_DECLS

#endif /* __IPC_GIT_CONFIG_H__ */
