
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#pragma once

#include <glib-object.h>
#include <libide-core.h>

G_BEGIN_DECLS

/* enumerations from "ide-animation.h" */

_IDE_EXTERN
GType ide_animation_mode_get_type (void);
#define IDE_TYPE_ANIMATION_MODE (ide_animation_mode_get_type())

/* enumerations from "ide-scrubber-revealer.h" */

_IDE_EXTERN
GType ide_scrubber_reveal_policy_get_type (void);
#define IDE_TYPE_SCRUBBER_REVEAL_POLICY (ide_scrubber_reveal_policy_get_type())

G_END_DECLS

/* Generated data ends here */

