
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#include "config.h"
#include "ide-code-enums.h"
#include "ide-buffer.h"
#include "ide-buffer-manager.h"
#include "ide-diagnostic.h"
#include "ide-indent-style.h"
#include "ide-spaces-style.h"
#include "ide-symbol.h"

#define C_ENUM(v) ((gint) v)
#define C_FLAGS(v) ((guint) v)

/* enumerations from "ide-buffer-manager.h" */

GType
ide_buffer_open_flags_get_type (void)
{
    static gsize gtype_id = 0;
    static const GFlagsValue values[] = {
        { C_FLAGS (IDE_BUFFER_OPEN_FLAGS_NONE), "IDE_BUFFER_OPEN_FLAGS_NONE", "none" },
        { C_FLAGS (IDE_BUFFER_OPEN_FLAGS_FORCE_RELOAD), "IDE_BUFFER_OPEN_FLAGS_FORCE_RELOAD", "force-reload" },
        { C_FLAGS (IDE_BUFFER_OPEN_FLAGS_DISABLE_ADDINS), "IDE_BUFFER_OPEN_FLAGS_DISABLE_ADDINS", "disable-addins" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_flags_register_static (g_intern_static_string ("IdeBufferOpenFlags"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "ide-buffer.h" */

GType
ide_buffer_state_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (IDE_BUFFER_STATE_READY), "IDE_BUFFER_STATE_READY", "ready" },
        { C_ENUM (IDE_BUFFER_STATE_LOADING), "IDE_BUFFER_STATE_LOADING", "loading" },
        { C_ENUM (IDE_BUFFER_STATE_SAVING), "IDE_BUFFER_STATE_SAVING", "saving" },
        { C_ENUM (IDE_BUFFER_STATE_FAILED), "IDE_BUFFER_STATE_FAILED", "failed" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("IdeBufferState"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "ide-diagnostic.h" */

GType
ide_diagnostic_severity_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (IDE_DIAGNOSTIC_IGNORED), "IDE_DIAGNOSTIC_IGNORED", "ignored" },
        { C_ENUM (IDE_DIAGNOSTIC_NOTE), "IDE_DIAGNOSTIC_NOTE", "note" },
        { C_ENUM (IDE_DIAGNOSTIC_UNUSED), "IDE_DIAGNOSTIC_UNUSED", "unused" },
        { C_ENUM (IDE_DIAGNOSTIC_DEPRECATED), "IDE_DIAGNOSTIC_DEPRECATED", "deprecated" },
        { C_ENUM (IDE_DIAGNOSTIC_WARNING), "IDE_DIAGNOSTIC_WARNING", "warning" },
        { C_ENUM (IDE_DIAGNOSTIC_ERROR), "IDE_DIAGNOSTIC_ERROR", "error" },
        { C_ENUM (IDE_DIAGNOSTIC_FATAL), "IDE_DIAGNOSTIC_FATAL", "fatal" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("IdeDiagnosticSeverity"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "ide-indent-style.h" */

GType
ide_indent_style_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (IDE_INDENT_STYLE_SPACES), "IDE_INDENT_STYLE_SPACES", "spaces" },
        { C_ENUM (IDE_INDENT_STYLE_TABS), "IDE_INDENT_STYLE_TABS", "tabs" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("IdeIndentStyle"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "ide-spaces-style.h" */

GType
ide_spaces_style_get_type (void)
{
    static gsize gtype_id = 0;
    static const GFlagsValue values[] = {
        { C_FLAGS (IDE_SPACES_STYLE_IGNORE), "IDE_SPACES_STYLE_IGNORE", "ignore" },
        { C_FLAGS (IDE_SPACES_STYLE_BEFORE_LEFT_PAREN), "IDE_SPACES_STYLE_BEFORE_LEFT_PAREN", "before-left-paren" },
        { C_FLAGS (IDE_SPACES_STYLE_BEFORE_LEFT_BRACKET), "IDE_SPACES_STYLE_BEFORE_LEFT_BRACKET", "before-left-bracket" },
        { C_FLAGS (IDE_SPACES_STYLE_BEFORE_LEFT_BRACE), "IDE_SPACES_STYLE_BEFORE_LEFT_BRACE", "before-left-brace" },
        { C_FLAGS (IDE_SPACES_STYLE_BEFORE_LEFT_ANGLE), "IDE_SPACES_STYLE_BEFORE_LEFT_ANGLE", "before-left-angle" },
        { C_FLAGS (IDE_SPACES_STYLE_BEFORE_COLON), "IDE_SPACES_STYLE_BEFORE_COLON", "before-colon" },
        { C_FLAGS (IDE_SPACES_STYLE_BEFORE_COMMA), "IDE_SPACES_STYLE_BEFORE_COMMA", "before-comma" },
        { C_FLAGS (IDE_SPACES_STYLE_BEFORE_SEMICOLON), "IDE_SPACES_STYLE_BEFORE_SEMICOLON", "before-semicolon" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_flags_register_static (g_intern_static_string ("IdeSpacesStyle"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "ide-symbol.h" */

GType
ide_symbol_kind_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (IDE_SYMBOL_KIND_NONE), "IDE_SYMBOL_KIND_NONE", "none" },
        { C_ENUM (IDE_SYMBOL_KIND_ALIAS), "IDE_SYMBOL_KIND_ALIAS", "alias" },
        { C_ENUM (IDE_SYMBOL_KIND_ARRAY), "IDE_SYMBOL_KIND_ARRAY", "array" },
        { C_ENUM (IDE_SYMBOL_KIND_BOOLEAN), "IDE_SYMBOL_KIND_BOOLEAN", "boolean" },
        { C_ENUM (IDE_SYMBOL_KIND_CLASS), "IDE_SYMBOL_KIND_CLASS", "class" },
        { C_ENUM (IDE_SYMBOL_KIND_CONSTANT), "IDE_SYMBOL_KIND_CONSTANT", "constant" },
        { C_ENUM (IDE_SYMBOL_KIND_CONSTRUCTOR), "IDE_SYMBOL_KIND_CONSTRUCTOR", "constructor" },
        { C_ENUM (IDE_SYMBOL_KIND_ENUM), "IDE_SYMBOL_KIND_ENUM", "enum" },
        { C_ENUM (IDE_SYMBOL_KIND_ENUM_VALUE), "IDE_SYMBOL_KIND_ENUM_VALUE", "enum-value" },
        { C_ENUM (IDE_SYMBOL_KIND_FIELD), "IDE_SYMBOL_KIND_FIELD", "field" },
        { C_ENUM (IDE_SYMBOL_KIND_FILE), "IDE_SYMBOL_KIND_FILE", "file" },
        { C_ENUM (IDE_SYMBOL_KIND_FUNCTION), "IDE_SYMBOL_KIND_FUNCTION", "function" },
        { C_ENUM (IDE_SYMBOL_KIND_HEADER), "IDE_SYMBOL_KIND_HEADER", "header" },
        { C_ENUM (IDE_SYMBOL_KIND_INTERFACE), "IDE_SYMBOL_KIND_INTERFACE", "interface" },
        { C_ENUM (IDE_SYMBOL_KIND_MACRO), "IDE_SYMBOL_KIND_MACRO", "macro" },
        { C_ENUM (IDE_SYMBOL_KIND_METHOD), "IDE_SYMBOL_KIND_METHOD", "method" },
        { C_ENUM (IDE_SYMBOL_KIND_MODULE), "IDE_SYMBOL_KIND_MODULE", "module" },
        { C_ENUM (IDE_SYMBOL_KIND_NAMESPACE), "IDE_SYMBOL_KIND_NAMESPACE", "namespace" },
        { C_ENUM (IDE_SYMBOL_KIND_NUMBER), "IDE_SYMBOL_KIND_NUMBER", "number" },
        { C_ENUM (IDE_SYMBOL_KIND_PACKAGE), "IDE_SYMBOL_KIND_PACKAGE", "package" },
        { C_ENUM (IDE_SYMBOL_KIND_PROPERTY), "IDE_SYMBOL_KIND_PROPERTY", "property" },
        { C_ENUM (IDE_SYMBOL_KIND_SCALAR), "IDE_SYMBOL_KIND_SCALAR", "scalar" },
        { C_ENUM (IDE_SYMBOL_KIND_STRING), "IDE_SYMBOL_KIND_STRING", "string" },
        { C_ENUM (IDE_SYMBOL_KIND_STRUCT), "IDE_SYMBOL_KIND_STRUCT", "struct" },
        { C_ENUM (IDE_SYMBOL_KIND_TEMPLATE), "IDE_SYMBOL_KIND_TEMPLATE", "template" },
        { C_ENUM (IDE_SYMBOL_KIND_UNION), "IDE_SYMBOL_KIND_UNION", "union" },
        { C_ENUM (IDE_SYMBOL_KIND_VARIABLE), "IDE_SYMBOL_KIND_VARIABLE", "variable" },
        { C_ENUM (IDE_SYMBOL_KIND_KEYWORD), "IDE_SYMBOL_KIND_KEYWORD", "keyword" },
        { C_ENUM (IDE_SYMBOL_KIND_UI_ATTRIBUTES), "IDE_SYMBOL_KIND_UI_ATTRIBUTES", "ui-attributes" },
        { C_ENUM (IDE_SYMBOL_KIND_UI_CHILD), "IDE_SYMBOL_KIND_UI_CHILD", "ui-child" },
        { C_ENUM (IDE_SYMBOL_KIND_UI_ITEM), "IDE_SYMBOL_KIND_UI_ITEM", "ui-item" },
        { C_ENUM (IDE_SYMBOL_KIND_UI_MENU), "IDE_SYMBOL_KIND_UI_MENU", "ui-menu" },
        { C_ENUM (IDE_SYMBOL_KIND_UI_MENU_ATTRIBUTE), "IDE_SYMBOL_KIND_UI_MENU_ATTRIBUTE", "ui-menu-attribute" },
        { C_ENUM (IDE_SYMBOL_KIND_UI_OBJECT), "IDE_SYMBOL_KIND_UI_OBJECT", "ui-object" },
        { C_ENUM (IDE_SYMBOL_KIND_UI_PACKING), "IDE_SYMBOL_KIND_UI_PACKING", "ui-packing" },
        { C_ENUM (IDE_SYMBOL_KIND_UI_PROPERTY), "IDE_SYMBOL_KIND_UI_PROPERTY", "ui-property" },
        { C_ENUM (IDE_SYMBOL_KIND_UI_SECTION), "IDE_SYMBOL_KIND_UI_SECTION", "ui-section" },
        { C_ENUM (IDE_SYMBOL_KIND_UI_SIGNAL), "IDE_SYMBOL_KIND_UI_SIGNAL", "ui-signal" },
        { C_ENUM (IDE_SYMBOL_KIND_UI_STYLE), "IDE_SYMBOL_KIND_UI_STYLE", "ui-style" },
        { C_ENUM (IDE_SYMBOL_KIND_UI_STYLE_CLASS), "IDE_SYMBOL_KIND_UI_STYLE_CLASS", "ui-style-class" },
        { C_ENUM (IDE_SYMBOL_KIND_UI_SUBMENU), "IDE_SYMBOL_KIND_UI_SUBMENU", "ui-submenu" },
        { C_ENUM (IDE_SYMBOL_KIND_UI_TEMPLATE), "IDE_SYMBOL_KIND_UI_TEMPLATE", "ui-template" },
        { C_ENUM (IDE_SYMBOL_KIND_XML_ATTRIBUTE), "IDE_SYMBOL_KIND_XML_ATTRIBUTE", "xml-attribute" },
        { C_ENUM (IDE_SYMBOL_KIND_XML_DECLARATION), "IDE_SYMBOL_KIND_XML_DECLARATION", "xml-declaration" },
        { C_ENUM (IDE_SYMBOL_KIND_XML_ELEMENT), "IDE_SYMBOL_KIND_XML_ELEMENT", "xml-element" },
        { C_ENUM (IDE_SYMBOL_KIND_XML_COMMENT), "IDE_SYMBOL_KIND_XML_COMMENT", "xml-comment" },
        { C_ENUM (IDE_SYMBOL_KIND_XML_CDATA), "IDE_SYMBOL_KIND_XML_CDATA", "xml-cdata" },
        { C_ENUM (IDE_SYMBOL_KIND_OBJECT), "IDE_SYMBOL_KIND_OBJECT", "object" },
        { C_ENUM (IDE_SYMBOL_KIND_EVENT), "IDE_SYMBOL_KIND_EVENT", "event" },
        { C_ENUM (IDE_SYMBOL_KIND_OPERATOR), "IDE_SYMBOL_KIND_OPERATOR", "operator" },
        { C_ENUM (IDE_SYMBOL_KIND_TYPE_PARAM), "IDE_SYMBOL_KIND_TYPE_PARAM", "type-param" },
        { C_ENUM (IDE_SYMBOL_KIND_LAST), "IDE_SYMBOL_KIND_LAST", "last" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("IdeSymbolKind"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
ide_symbol_flags_get_type (void)
{
    static gsize gtype_id = 0;
    static const GFlagsValue values[] = {
        { C_FLAGS (IDE_SYMBOL_FLAGS_NONE), "IDE_SYMBOL_FLAGS_NONE", "none" },
        { C_FLAGS (IDE_SYMBOL_FLAGS_IS_STATIC), "IDE_SYMBOL_FLAGS_IS_STATIC", "is-static" },
        { C_FLAGS (IDE_SYMBOL_FLAGS_IS_MEMBER), "IDE_SYMBOL_FLAGS_IS_MEMBER", "is-member" },
        { C_FLAGS (IDE_SYMBOL_FLAGS_IS_DEPRECATED), "IDE_SYMBOL_FLAGS_IS_DEPRECATED", "is-deprecated" },
        { C_FLAGS (IDE_SYMBOL_FLAGS_IS_DEFINITION), "IDE_SYMBOL_FLAGS_IS_DEFINITION", "is-definition" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_flags_register_static (g_intern_static_string ("IdeSymbolFlags"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* Generated data ends here */

