
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#include "config.h"
#include "ide-search-enums.h"
#include "ide-search-provider.h"

#define C_ENUM(v) ((gint) v)
#define C_FLAGS(v) ((guint) v)

/* enumerations from "ide-search-provider.h" */

GType
ide_search_category_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (IDE_SEARCH_CATEGORY_EVERYTHING), "IDE_SEARCH_CATEGORY_EVERYTHING", "everything" },
        { C_ENUM (IDE_SEARCH_CATEGORY_ACTIONS), "IDE_SEARCH_CATEGORY_ACTIONS", "actions" },
        { C_ENUM (IDE_SEARCH_CATEGORY_COMMANDS), "IDE_SEARCH_CATEGORY_COMMANDS", "commands" },
        { C_ENUM (IDE_SEARCH_CATEGORY_FILES), "IDE_SEARCH_CATEGORY_FILES", "files" },
        { C_ENUM (IDE_SEARCH_CATEGORY_SYMBOLS), "IDE_SEARCH_CATEGORY_SYMBOLS", "symbols" },
        { C_ENUM (IDE_SEARCH_CATEGORY_OTHER), "IDE_SEARCH_CATEGORY_OTHER", "other" },
        { C_ENUM (IDE_SEARCH_CATEGORY_DOCUMENTATION), "IDE_SEARCH_CATEGORY_DOCUMENTATION", "documentation" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("IdeSearchCategory"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* Generated data ends here */

