/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __IPC_GIT_CHANGE_MONITOR_H__
#define __IPC_GIT_CHANGE_MONITOR_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.gnome.Builder.Git.ChangeMonitor */

#define IPC_TYPE_GIT_CHANGE_MONITOR (ipc_git_change_monitor_get_type ())
#define IPC_GIT_CHANGE_MONITOR(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), IPC_TYPE_GIT_CHANGE_MONITOR, IpcGitChangeMonitor))
#define IPC_IS_GIT_CHANGE_MONITOR(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), IPC_TYPE_GIT_CHANGE_MONITOR))
#define IPC_GIT_CHANGE_MONITOR_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), IPC_TYPE_GIT_CHANGE_MONITOR, IpcGitChangeMonitorIface))

struct _IpcGitChangeMonitor;
typedef struct _IpcGitChangeMonitor IpcGitChangeMonitor;
typedef struct _IpcGitChangeMonitorIface IpcGitChangeMonitorIface;

struct _IpcGitChangeMonitorIface
{
  GTypeInterface parent_iface;



  gboolean (*handle_close) (
    IpcGitChangeMonitor *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_list_changes) (
    IpcGitChangeMonitor *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_update_content) (
    IpcGitChangeMonitor *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_contents);

  const gchar * (*get_path) (IpcGitChangeMonitor *object);

  void (*closed) (
    IpcGitChangeMonitor *object);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (IpcGitChangeMonitor, g_object_unref)
#endif

GType ipc_git_change_monitor_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *ipc_git_change_monitor_interface_info (void);
guint ipc_git_change_monitor_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void ipc_git_change_monitor_complete_update_content (
    IpcGitChangeMonitor *object,
    GDBusMethodInvocation *invocation);

void ipc_git_change_monitor_complete_list_changes (
    IpcGitChangeMonitor *object,
    GDBusMethodInvocation *invocation,
    GVariant *changes);

void ipc_git_change_monitor_complete_close (
    IpcGitChangeMonitor *object,
    GDBusMethodInvocation *invocation);



/* D-Bus signal emissions functions: */
void ipc_git_change_monitor_emit_closed (
    IpcGitChangeMonitor *object);



/* D-Bus method calls: */
void ipc_git_change_monitor_call_update_content (
    IpcGitChangeMonitor *proxy,
    const gchar *arg_contents,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean ipc_git_change_monitor_call_update_content_finish (
    IpcGitChangeMonitor *proxy,
    GAsyncResult *res,
    GError **error);

gboolean ipc_git_change_monitor_call_update_content_sync (
    IpcGitChangeMonitor *proxy,
    const gchar *arg_contents,
    GCancellable *cancellable,
    GError **error);

void ipc_git_change_monitor_call_list_changes (
    IpcGitChangeMonitor *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean ipc_git_change_monitor_call_list_changes_finish (
    IpcGitChangeMonitor *proxy,
    GVariant ** out_changes,
    GAsyncResult *res,
    GError **error);

gboolean ipc_git_change_monitor_call_list_changes_sync (
    IpcGitChangeMonitor *proxy,
    GVariant ** out_changes,
    GCancellable *cancellable,
    GError **error);

void ipc_git_change_monitor_call_close (
    IpcGitChangeMonitor *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean ipc_git_change_monitor_call_close_finish (
    IpcGitChangeMonitor *proxy,
    GAsyncResult *res,
    GError **error);

gboolean ipc_git_change_monitor_call_close_sync (
    IpcGitChangeMonitor *proxy,
    GCancellable *cancellable,
    GError **error);



/* D-Bus property accessors: */
const gchar *ipc_git_change_monitor_get_path (IpcGitChangeMonitor *object);
gchar *ipc_git_change_monitor_dup_path (IpcGitChangeMonitor *object);
void ipc_git_change_monitor_set_path (IpcGitChangeMonitor *object, const gchar *value);


/* ---- */

#define IPC_TYPE_GIT_CHANGE_MONITOR_PROXY (ipc_git_change_monitor_proxy_get_type ())
#define IPC_GIT_CHANGE_MONITOR_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), IPC_TYPE_GIT_CHANGE_MONITOR_PROXY, IpcGitChangeMonitorProxy))
#define IPC_GIT_CHANGE_MONITOR_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), IPC_TYPE_GIT_CHANGE_MONITOR_PROXY, IpcGitChangeMonitorProxyClass))
#define IPC_GIT_CHANGE_MONITOR_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), IPC_TYPE_GIT_CHANGE_MONITOR_PROXY, IpcGitChangeMonitorProxyClass))
#define IPC_IS_GIT_CHANGE_MONITOR_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), IPC_TYPE_GIT_CHANGE_MONITOR_PROXY))
#define IPC_IS_GIT_CHANGE_MONITOR_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), IPC_TYPE_GIT_CHANGE_MONITOR_PROXY))

typedef struct _IpcGitChangeMonitorProxy IpcGitChangeMonitorProxy;
typedef struct _IpcGitChangeMonitorProxyClass IpcGitChangeMonitorProxyClass;
typedef struct _IpcGitChangeMonitorProxyPrivate IpcGitChangeMonitorProxyPrivate;

struct _IpcGitChangeMonitorProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  IpcGitChangeMonitorProxyPrivate *priv;
};

struct _IpcGitChangeMonitorProxyClass
{
  GDBusProxyClass parent_class;
};

GType ipc_git_change_monitor_proxy_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (IpcGitChangeMonitorProxy, g_object_unref)
#endif

void ipc_git_change_monitor_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
IpcGitChangeMonitor *ipc_git_change_monitor_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
IpcGitChangeMonitor *ipc_git_change_monitor_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void ipc_git_change_monitor_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
IpcGitChangeMonitor *ipc_git_change_monitor_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
IpcGitChangeMonitor *ipc_git_change_monitor_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define IPC_TYPE_GIT_CHANGE_MONITOR_SKELETON (ipc_git_change_monitor_skeleton_get_type ())
#define IPC_GIT_CHANGE_MONITOR_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), IPC_TYPE_GIT_CHANGE_MONITOR_SKELETON, IpcGitChangeMonitorSkeleton))
#define IPC_GIT_CHANGE_MONITOR_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), IPC_TYPE_GIT_CHANGE_MONITOR_SKELETON, IpcGitChangeMonitorSkeletonClass))
#define IPC_GIT_CHANGE_MONITOR_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), IPC_TYPE_GIT_CHANGE_MONITOR_SKELETON, IpcGitChangeMonitorSkeletonClass))
#define IPC_IS_GIT_CHANGE_MONITOR_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), IPC_TYPE_GIT_CHANGE_MONITOR_SKELETON))
#define IPC_IS_GIT_CHANGE_MONITOR_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), IPC_TYPE_GIT_CHANGE_MONITOR_SKELETON))

typedef struct _IpcGitChangeMonitorSkeleton IpcGitChangeMonitorSkeleton;
typedef struct _IpcGitChangeMonitorSkeletonClass IpcGitChangeMonitorSkeletonClass;
typedef struct _IpcGitChangeMonitorSkeletonPrivate IpcGitChangeMonitorSkeletonPrivate;

struct _IpcGitChangeMonitorSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  IpcGitChangeMonitorSkeletonPrivate *priv;
};

struct _IpcGitChangeMonitorSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType ipc_git_change_monitor_skeleton_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (IpcGitChangeMonitorSkeleton, g_object_unref)
#endif

IpcGitChangeMonitor *ipc_git_change_monitor_skeleton_new (void);


G_END_DECLS

#endif /* __IPC_GIT_CHANGE_MONITOR_H__ */
