
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#include "config.h"
#include "ide-foundry-enums.h"
#include "ide-build-log.h"
#include "ide-config.h"
#include "ide-device.h"
#include "ide-device-info.h"
#include "ide-pipeline-phase.h"
#include "ide-run-command.h"
#include "ide-run-context.h"
#include "ide-runtime.h"
#include "ide-test.h"

#define C_ENUM(v) ((gint) v)
#define C_FLAGS(v) ((guint) v)

/* enumerations from "ide-build-log.h" */

GType
ide_build_log_stream_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (IDE_BUILD_LOG_STDOUT), "IDE_BUILD_LOG_STDOUT", "stdout" },
        { C_ENUM (IDE_BUILD_LOG_STDERR), "IDE_BUILD_LOG_STDERR", "stderr" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("IdeBuildLogStream"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "ide-config.h" */

GType
ide_build_locality_get_type (void)
{
    static gsize gtype_id = 0;
    static const GFlagsValue values[] = {
        { C_FLAGS (IDE_BUILD_LOCALITY_IN_TREE), "IDE_BUILD_LOCALITY_IN_TREE", "in-tree" },
        { C_FLAGS (IDE_BUILD_LOCALITY_OUT_OF_TREE), "IDE_BUILD_LOCALITY_OUT_OF_TREE", "out-of-tree" },
        { C_FLAGS (IDE_BUILD_LOCALITY_DEFAULT), "IDE_BUILD_LOCALITY_DEFAULT", "default" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_flags_register_static (g_intern_static_string ("IdeBuildLocality"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "ide-device-info.h" */

GType
ide_device_kind_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (IDE_DEVICE_KIND_COMPUTER), "IDE_DEVICE_KIND_COMPUTER", "computer" },
        { C_ENUM (IDE_DEVICE_KIND_PHONE), "IDE_DEVICE_KIND_PHONE", "phone" },
        { C_ENUM (IDE_DEVICE_KIND_TABLET), "IDE_DEVICE_KIND_TABLET", "tablet" },
        { C_ENUM (IDE_DEVICE_KIND_MICRO_CONTROLLER), "IDE_DEVICE_KIND_MICRO_CONTROLLER", "micro-controller" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("IdeDeviceKind"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "ide-device.h" */

GType
ide_device_error_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (IDE_DEVICE_ERROR_NO_SUCH_DEVICE), "IDE_DEVICE_ERROR_NO_SUCH_DEVICE", "device" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("IdeDeviceError"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "ide-pipeline-phase.h" */

GType
ide_pipeline_phase_get_type (void)
{
    static gsize gtype_id = 0;
    static const GFlagsValue values[] = {
        { C_FLAGS (IDE_PIPELINE_PHASE_NONE), "IDE_PIPELINE_PHASE_NONE", "none" },
        { C_FLAGS (IDE_PIPELINE_PHASE_PREPARE), "IDE_PIPELINE_PHASE_PREPARE", "prepare" },
        { C_FLAGS (IDE_PIPELINE_PHASE_DOWNLOADS), "IDE_PIPELINE_PHASE_DOWNLOADS", "downloads" },
        { C_FLAGS (IDE_PIPELINE_PHASE_DEPENDENCIES), "IDE_PIPELINE_PHASE_DEPENDENCIES", "dependencies" },
        { C_FLAGS (IDE_PIPELINE_PHASE_AUTOGEN), "IDE_PIPELINE_PHASE_AUTOGEN", "autogen" },
        { C_FLAGS (IDE_PIPELINE_PHASE_CONFIGURE), "IDE_PIPELINE_PHASE_CONFIGURE", "configure" },
        { C_FLAGS (IDE_PIPELINE_PHASE_BUILD), "IDE_PIPELINE_PHASE_BUILD", "build" },
        { C_FLAGS (IDE_PIPELINE_PHASE_INSTALL), "IDE_PIPELINE_PHASE_INSTALL", "install" },
        { C_FLAGS (IDE_PIPELINE_PHASE_COMMIT), "IDE_PIPELINE_PHASE_COMMIT", "commit" },
        { C_FLAGS (IDE_PIPELINE_PHASE_EXPORT), "IDE_PIPELINE_PHASE_EXPORT", "export" },
        { C_FLAGS (IDE_PIPELINE_PHASE_FINAL), "IDE_PIPELINE_PHASE_FINAL", "final" },
        { C_FLAGS (IDE_PIPELINE_PHASE_BEFORE), "IDE_PIPELINE_PHASE_BEFORE", "before" },
        { C_FLAGS (IDE_PIPELINE_PHASE_AFTER), "IDE_PIPELINE_PHASE_AFTER", "after" },
        { C_FLAGS (IDE_PIPELINE_PHASE_FINISHED), "IDE_PIPELINE_PHASE_FINISHED", "finished" },
        { C_FLAGS (IDE_PIPELINE_PHASE_FAILED), "IDE_PIPELINE_PHASE_FAILED", "failed" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_flags_register_static (g_intern_static_string ("IdePipelinePhase"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "ide-run-command.h" */

GType
ide_run_command_kind_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (IDE_RUN_COMMAND_KIND_UNKNOWN), "IDE_RUN_COMMAND_KIND_UNKNOWN", "unknown" },
        { C_ENUM (IDE_RUN_COMMAND_KIND_APPLICATION), "IDE_RUN_COMMAND_KIND_APPLICATION", "application" },
        { C_ENUM (IDE_RUN_COMMAND_KIND_UTILITY), "IDE_RUN_COMMAND_KIND_UTILITY", "utility" },
        { C_ENUM (IDE_RUN_COMMAND_KIND_TEST), "IDE_RUN_COMMAND_KIND_TEST", "test" },
        { C_ENUM (IDE_RUN_COMMAND_KIND_BENCHMARK), "IDE_RUN_COMMAND_KIND_BENCHMARK", "benchmark" },
        { C_ENUM (IDE_RUN_COMMAND_KIND_USER_DEFINED), "IDE_RUN_COMMAND_KIND_USER_DEFINED", "user-defined" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("IdeRunCommandKind"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "ide-run-context.h" */

GType
ide_run_context_shell_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (IDE_RUN_CONTEXT_SHELL_DEFAULT), "IDE_RUN_CONTEXT_SHELL_DEFAULT", "default" },
        { C_ENUM (IDE_RUN_CONTEXT_SHELL_LOGIN), "IDE_RUN_CONTEXT_SHELL_LOGIN", "login" },
        { C_ENUM (IDE_RUN_CONTEXT_SHELL_INTERACTIVE), "IDE_RUN_CONTEXT_SHELL_INTERACTIVE", "interactive" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("IdeRunContextShell"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "ide-runtime.h" */

GType
ide_runtime_error_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (IDE_RUNTIME_ERROR_UNKNOWN), "IDE_RUNTIME_ERROR_UNKNOWN", "unknown" },
        { C_ENUM (IDE_RUNTIME_ERROR_NO_SUCH_RUNTIME), "IDE_RUNTIME_ERROR_NO_SUCH_RUNTIME", "no-such-runtime" },
        { C_ENUM (IDE_RUNTIME_ERROR_BUILD_FAILED), "IDE_RUNTIME_ERROR_BUILD_FAILED", "build-failed" },
        { C_ENUM (IDE_RUNTIME_ERROR_TARGET_NOT_FOUND), "IDE_RUNTIME_ERROR_TARGET_NOT_FOUND", "target-not-found" },
        { C_ENUM (IDE_RUNTIME_ERROR_SPAWN_FAILED), "IDE_RUNTIME_ERROR_SPAWN_FAILED", "spawn-failed" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("IdeRuntimeError"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "ide-test.h" */

GType
ide_test_status_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (IDE_TEST_STATUS_NONE), "IDE_TEST_STATUS_NONE", "none" },
        { C_ENUM (IDE_TEST_STATUS_RUNNING), "IDE_TEST_STATUS_RUNNING", "running" },
        { C_ENUM (IDE_TEST_STATUS_SUCCESS), "IDE_TEST_STATUS_SUCCESS", "success" },
        { C_ENUM (IDE_TEST_STATUS_FAILED), "IDE_TEST_STATUS_FAILED", "failed" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("IdeTestStatus"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* Generated data ends here */

