/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __IPC_GIT_BLAME_H__
#define __IPC_GIT_BLAME_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.gnome.Builder.Git.Blame */

#define IPC_TYPE_GIT_BLAME (ipc_git_blame_get_type ())
#define IPC_GIT_BLAME(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), IPC_TYPE_GIT_BLAME, IpcGitBlame))
#define IPC_IS_GIT_BLAME(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), IPC_TYPE_GIT_BLAME))
#define IPC_GIT_BLAME_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), IPC_TYPE_GIT_BLAME, IpcGitBlameIface))

struct _IpcGitBlame;
typedef struct _IpcGitBlame IpcGitBlame;
typedef struct _IpcGitBlameIface IpcGitBlameIface;

struct _IpcGitBlameIface
{
  GTypeInterface parent_iface;



  gboolean (*handle_query_line) (
    IpcGitBlame *object,
    GDBusMethodInvocation *invocation,
    guint arg_line_number);

  gboolean (*handle_query_line_range) (
    IpcGitBlame *object,
    GDBusMethodInvocation *invocation,
    guint arg_first,
    guint arg_lenght);

  gboolean (*handle_update_content) (
    IpcGitBlame *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_contents);

  const gchar * (*get_path) (IpcGitBlame *object);

  void (*closed) (
    IpcGitBlame *object);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (IpcGitBlame, g_object_unref)
#endif

GType ipc_git_blame_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *ipc_git_blame_interface_info (void);
guint ipc_git_blame_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void ipc_git_blame_complete_update_content (
    IpcGitBlame *object,
    GDBusMethodInvocation *invocation);

void ipc_git_blame_complete_query_line (
    IpcGitBlame *object,
    GDBusMethodInvocation *invocation,
    const gchar *commit_hash,
    const gchar *author_name,
    const gchar *author_email,
    const gchar *commit_message,
    const gchar *commit_date,
    guint line_in_commit);

void ipc_git_blame_complete_query_line_range (
    IpcGitBlame *object,
    GDBusMethodInvocation *invocation,
    GVariant *blame_range);



/* D-Bus signal emissions functions: */
void ipc_git_blame_emit_closed (
    IpcGitBlame *object);



/* D-Bus method calls: */
void ipc_git_blame_call_update_content (
    IpcGitBlame *proxy,
    const gchar *arg_contents,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean ipc_git_blame_call_update_content_finish (
    IpcGitBlame *proxy,
    GAsyncResult *res,
    GError **error);

gboolean ipc_git_blame_call_update_content_sync (
    IpcGitBlame *proxy,
    const gchar *arg_contents,
    GCancellable *cancellable,
    GError **error);

void ipc_git_blame_call_query_line (
    IpcGitBlame *proxy,
    guint arg_line_number,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean ipc_git_blame_call_query_line_finish (
    IpcGitBlame *proxy,
    gchar ** out_commit_hash,
    gchar ** out_author_name,
    gchar ** out_author_email,
    gchar ** out_commit_message,
    gchar ** out_commit_date,
    guint* out_line_in_commit,
    GAsyncResult *res,
    GError **error);

gboolean ipc_git_blame_call_query_line_sync (
    IpcGitBlame *proxy,
    guint arg_line_number,
    gchar ** out_commit_hash,
    gchar ** out_author_name,
    gchar ** out_author_email,
    gchar ** out_commit_message,
    gchar ** out_commit_date,
    guint* out_line_in_commit,
    GCancellable *cancellable,
    GError **error);

void ipc_git_blame_call_query_line_range (
    IpcGitBlame *proxy,
    guint arg_first,
    guint arg_lenght,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean ipc_git_blame_call_query_line_range_finish (
    IpcGitBlame *proxy,
    GVariant ** out_blame_range,
    GAsyncResult *res,
    GError **error);

gboolean ipc_git_blame_call_query_line_range_sync (
    IpcGitBlame *proxy,
    guint arg_first,
    guint arg_lenght,
    GVariant ** out_blame_range,
    GCancellable *cancellable,
    GError **error);



/* D-Bus property accessors: */
const gchar *ipc_git_blame_get_path (IpcGitBlame *object);
gchar *ipc_git_blame_dup_path (IpcGitBlame *object);
void ipc_git_blame_set_path (IpcGitBlame *object, const gchar *value);


/* ---- */

#define IPC_TYPE_GIT_BLAME_PROXY (ipc_git_blame_proxy_get_type ())
#define IPC_GIT_BLAME_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), IPC_TYPE_GIT_BLAME_PROXY, IpcGitBlameProxy))
#define IPC_GIT_BLAME_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), IPC_TYPE_GIT_BLAME_PROXY, IpcGitBlameProxyClass))
#define IPC_GIT_BLAME_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), IPC_TYPE_GIT_BLAME_PROXY, IpcGitBlameProxyClass))
#define IPC_IS_GIT_BLAME_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), IPC_TYPE_GIT_BLAME_PROXY))
#define IPC_IS_GIT_BLAME_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), IPC_TYPE_GIT_BLAME_PROXY))

typedef struct _IpcGitBlameProxy IpcGitBlameProxy;
typedef struct _IpcGitBlameProxyClass IpcGitBlameProxyClass;
typedef struct _IpcGitBlameProxyPrivate IpcGitBlameProxyPrivate;

struct _IpcGitBlameProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  IpcGitBlameProxyPrivate *priv;
};

struct _IpcGitBlameProxyClass
{
  GDBusProxyClass parent_class;
};

GType ipc_git_blame_proxy_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (IpcGitBlameProxy, g_object_unref)
#endif

void ipc_git_blame_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
IpcGitBlame *ipc_git_blame_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
IpcGitBlame *ipc_git_blame_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void ipc_git_blame_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
IpcGitBlame *ipc_git_blame_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
IpcGitBlame *ipc_git_blame_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define IPC_TYPE_GIT_BLAME_SKELETON (ipc_git_blame_skeleton_get_type ())
#define IPC_GIT_BLAME_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), IPC_TYPE_GIT_BLAME_SKELETON, IpcGitBlameSkeleton))
#define IPC_GIT_BLAME_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), IPC_TYPE_GIT_BLAME_SKELETON, IpcGitBlameSkeletonClass))
#define IPC_GIT_BLAME_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), IPC_TYPE_GIT_BLAME_SKELETON, IpcGitBlameSkeletonClass))
#define IPC_IS_GIT_BLAME_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), IPC_TYPE_GIT_BLAME_SKELETON))
#define IPC_IS_GIT_BLAME_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), IPC_TYPE_GIT_BLAME_SKELETON))

typedef struct _IpcGitBlameSkeleton IpcGitBlameSkeleton;
typedef struct _IpcGitBlameSkeletonClass IpcGitBlameSkeletonClass;
typedef struct _IpcGitBlameSkeletonPrivate IpcGitBlameSkeletonPrivate;

struct _IpcGitBlameSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  IpcGitBlameSkeletonPrivate *priv;
};

struct _IpcGitBlameSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType ipc_git_blame_skeleton_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (IpcGitBlameSkeleton, g_object_unref)
#endif

IpcGitBlame *ipc_git_blame_skeleton_new (void);


G_END_DECLS

#endif /* __IPC_GIT_BLAME_H__ */
