/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __IPC_GIT_PROGRESS_H__
#define __IPC_GIT_PROGRESS_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.gnome.Builder.Git.Progress */

#define IPC_TYPE_GIT_PROGRESS (ipc_git_progress_get_type ())
#define IPC_GIT_PROGRESS(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), IPC_TYPE_GIT_PROGRESS, IpcGitProgress))
#define IPC_IS_GIT_PROGRESS(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), IPC_TYPE_GIT_PROGRESS))
#define IPC_GIT_PROGRESS_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), IPC_TYPE_GIT_PROGRESS, IpcGitProgressIface))

struct _IpcGitProgress;
typedef struct _IpcGitProgress IpcGitProgress;
typedef struct _IpcGitProgressIface IpcGitProgressIface;

struct _IpcGitProgressIface
{
  GTypeInterface parent_iface;

  gdouble  (*get_fraction) (IpcGitProgress *object);

  const gchar * (*get_message) (IpcGitProgress *object);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (IpcGitProgress, g_object_unref)
#endif

GType ipc_git_progress_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *ipc_git_progress_interface_info (void);
guint ipc_git_progress_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus property accessors: */
gdouble ipc_git_progress_get_fraction (IpcGitProgress *object);
void ipc_git_progress_set_fraction (IpcGitProgress *object, gdouble value);

const gchar *ipc_git_progress_get_message (IpcGitProgress *object);
gchar *ipc_git_progress_dup_message (IpcGitProgress *object);
void ipc_git_progress_set_message (IpcGitProgress *object, const gchar *value);


/* ---- */

#define IPC_TYPE_GIT_PROGRESS_PROXY (ipc_git_progress_proxy_get_type ())
#define IPC_GIT_PROGRESS_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), IPC_TYPE_GIT_PROGRESS_PROXY, IpcGitProgressProxy))
#define IPC_GIT_PROGRESS_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), IPC_TYPE_GIT_PROGRESS_PROXY, IpcGitProgressProxyClass))
#define IPC_GIT_PROGRESS_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), IPC_TYPE_GIT_PROGRESS_PROXY, IpcGitProgressProxyClass))
#define IPC_IS_GIT_PROGRESS_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), IPC_TYPE_GIT_PROGRESS_PROXY))
#define IPC_IS_GIT_PROGRESS_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), IPC_TYPE_GIT_PROGRESS_PROXY))

typedef struct _IpcGitProgressProxy IpcGitProgressProxy;
typedef struct _IpcGitProgressProxyClass IpcGitProgressProxyClass;
typedef struct _IpcGitProgressProxyPrivate IpcGitProgressProxyPrivate;

struct _IpcGitProgressProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  IpcGitProgressProxyPrivate *priv;
};

struct _IpcGitProgressProxyClass
{
  GDBusProxyClass parent_class;
};

GType ipc_git_progress_proxy_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (IpcGitProgressProxy, g_object_unref)
#endif

void ipc_git_progress_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
IpcGitProgress *ipc_git_progress_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
IpcGitProgress *ipc_git_progress_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void ipc_git_progress_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
IpcGitProgress *ipc_git_progress_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
IpcGitProgress *ipc_git_progress_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define IPC_TYPE_GIT_PROGRESS_SKELETON (ipc_git_progress_skeleton_get_type ())
#define IPC_GIT_PROGRESS_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), IPC_TYPE_GIT_PROGRESS_SKELETON, IpcGitProgressSkeleton))
#define IPC_GIT_PROGRESS_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), IPC_TYPE_GIT_PROGRESS_SKELETON, IpcGitProgressSkeletonClass))
#define IPC_GIT_PROGRESS_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), IPC_TYPE_GIT_PROGRESS_SKELETON, IpcGitProgressSkeletonClass))
#define IPC_IS_GIT_PROGRESS_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), IPC_TYPE_GIT_PROGRESS_SKELETON))
#define IPC_IS_GIT_PROGRESS_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), IPC_TYPE_GIT_PROGRESS_SKELETON))

typedef struct _IpcGitProgressSkeleton IpcGitProgressSkeleton;
typedef struct _IpcGitProgressSkeletonClass IpcGitProgressSkeletonClass;
typedef struct _IpcGitProgressSkeletonPrivate IpcGitProgressSkeletonPrivate;

struct _IpcGitProgressSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  IpcGitProgressSkeletonPrivate *priv;
};

struct _IpcGitProgressSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType ipc_git_progress_skeleton_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (IpcGitProgressSkeleton, g_object_unref)
#endif

IpcGitProgress *ipc_git_progress_skeleton_new (void);


G_END_DECLS

#endif /* __IPC_GIT_PROGRESS_H__ */
