/* This file is generated by glib-genmarshal, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */
#pragma once

#include <glib-object.h>

G_BEGIN_DECLS

/* BOOLEAN:OBJECT (../gnome-builder/src/libide/core/ide-marshal.defs:1) */
G_GNUC_INTERNAL
void ide_marshal_BOOLEAN__OBJECT (GClosure     *closure,
                                  GValue       *return_value,
                                  guint         n_param_values,
                                  const GValue *param_values,
                                  gpointer      invocation_hint,
                                  gpointer      marshal_data);
G_GNUC_INTERNAL
void ide_marshal_BOOLEAN__OBJECTv (GClosure *closure,
                                   GValue   *return_value,
                                   gpointer  instance,
                                   va_list   args,
                                   gpointer  marshal_data,
                                   int       n_params,
                                   GType    *param_types);

/* BOOLEAN:STRING (../gnome-builder/src/libide/core/ide-marshal.defs:2) */
G_GNUC_INTERNAL
void ide_marshal_BOOLEAN__STRING (GClosure     *closure,
                                  GValue       *return_value,
                                  guint         n_param_values,
                                  const GValue *param_values,
                                  gpointer      invocation_hint,
                                  gpointer      marshal_data);
G_GNUC_INTERNAL
void ide_marshal_BOOLEAN__STRINGv (GClosure *closure,
                                   GValue   *return_value,
                                   gpointer  instance,
                                   va_list   args,
                                   gpointer  marshal_data,
                                   int       n_params,
                                   GType    *param_types);

/* BOXED:VOID (../gnome-builder/src/libide/core/ide-marshal.defs:3) */
G_GNUC_INTERNAL
void ide_marshal_BOXED__VOID (GClosure     *closure,
                              GValue       *return_value,
                              guint         n_param_values,
                              const GValue *param_values,
                              gpointer      invocation_hint,
                              gpointer      marshal_data);
G_GNUC_INTERNAL
void ide_marshal_BOXED__VOIDv (GClosure *closure,
                               GValue   *return_value,
                               gpointer  instance,
                               va_list   args,
                               gpointer  marshal_data,
                               int       n_params,
                               GType    *param_types);

/* OBJECT:OBJECT (../gnome-builder/src/libide/core/ide-marshal.defs:4) */
G_GNUC_INTERNAL
void ide_marshal_OBJECT__OBJECT (GClosure     *closure,
                                 GValue       *return_value,
                                 guint         n_param_values,
                                 const GValue *param_values,
                                 gpointer      invocation_hint,
                                 gpointer      marshal_data);
G_GNUC_INTERNAL
void ide_marshal_OBJECT__OBJECTv (GClosure *closure,
                                  GValue   *return_value,
                                  gpointer  instance,
                                  va_list   args,
                                  gpointer  marshal_data,
                                  int       n_params,
                                  GType    *param_types);

/* VOID:BOOLEAN (../gnome-builder/src/libide/core/ide-marshal.defs:5) */
#define ide_marshal_VOID__BOOLEAN	g_cclosure_marshal_VOID__BOOLEAN
#define ide_marshal_VOID__BOOLEANv	g_cclosure_marshal_VOID__BOOLEANv

/* VOID:BOXED,OBJECT (../gnome-builder/src/libide/core/ide-marshal.defs:6) */
G_GNUC_INTERNAL
void ide_marshal_VOID__BOXED_OBJECT (GClosure     *closure,
                                     GValue       *return_value,
                                     guint         n_param_values,
                                     const GValue *param_values,
                                     gpointer      invocation_hint,
                                     gpointer      marshal_data);
G_GNUC_INTERNAL
void ide_marshal_VOID__BOXED_OBJECTv (GClosure *closure,
                                      GValue   *return_value,
                                      gpointer  instance,
                                      va_list   args,
                                      gpointer  marshal_data,
                                      int       n_params,
                                      GType    *param_types);

/* VOID:FLAGS (../gnome-builder/src/libide/core/ide-marshal.defs:7) */
#define ide_marshal_VOID__FLAGS	g_cclosure_marshal_VOID__FLAGS
#define ide_marshal_VOID__FLAGSv	g_cclosure_marshal_VOID__FLAGSv

/* VOID:OBJECT (../gnome-builder/src/libide/core/ide-marshal.defs:8) */
#define ide_marshal_VOID__OBJECT	g_cclosure_marshal_VOID__OBJECT
#define ide_marshal_VOID__OBJECTv	g_cclosure_marshal_VOID__OBJECTv

/* VOID:OBJECT,BOXED (../gnome-builder/src/libide/core/ide-marshal.defs:9) */
G_GNUC_INTERNAL
void ide_marshal_VOID__OBJECT_BOXED (GClosure     *closure,
                                     GValue       *return_value,
                                     guint         n_param_values,
                                     const GValue *param_values,
                                     gpointer      invocation_hint,
                                     gpointer      marshal_data);
G_GNUC_INTERNAL
void ide_marshal_VOID__OBJECT_BOXEDv (GClosure *closure,
                                      GValue   *return_value,
                                      gpointer  instance,
                                      va_list   args,
                                      gpointer  marshal_data,
                                      int       n_params,
                                      GType    *param_types);

/* VOID:OBJECT,BOXED,OBJECT (../gnome-builder/src/libide/core/ide-marshal.defs:10) */
G_GNUC_INTERNAL
void ide_marshal_VOID__OBJECT_BOXED_OBJECT (GClosure     *closure,
                                            GValue       *return_value,
                                            guint         n_param_values,
                                            const GValue *param_values,
                                            gpointer      invocation_hint,
                                            gpointer      marshal_data);
G_GNUC_INTERNAL
void ide_marshal_VOID__OBJECT_BOXED_OBJECTv (GClosure *closure,
                                             GValue   *return_value,
                                             gpointer  instance,
                                             va_list   args,
                                             gpointer  marshal_data,
                                             int       n_params,
                                             GType    *param_types);

/* VOID:OBJECT,OBJECT (../gnome-builder/src/libide/core/ide-marshal.defs:11) */
G_GNUC_INTERNAL
void ide_marshal_VOID__OBJECT_OBJECT (GClosure     *closure,
                                      GValue       *return_value,
                                      guint         n_param_values,
                                      const GValue *param_values,
                                      gpointer      invocation_hint,
                                      gpointer      marshal_data);
G_GNUC_INTERNAL
void ide_marshal_VOID__OBJECT_OBJECTv (GClosure *closure,
                                       GValue   *return_value,
                                       gpointer  instance,
                                       va_list   args,
                                       gpointer  marshal_data,
                                       int       n_params,
                                       GType    *param_types);

/* VOID:OBJECT,OBJECT,ENUM (../gnome-builder/src/libide/core/ide-marshal.defs:12) */
G_GNUC_INTERNAL
void ide_marshal_VOID__OBJECT_OBJECT_ENUM (GClosure     *closure,
                                           GValue       *return_value,
                                           guint         n_param_values,
                                           const GValue *param_values,
                                           gpointer      invocation_hint,
                                           gpointer      marshal_data);
G_GNUC_INTERNAL
void ide_marshal_VOID__OBJECT_OBJECT_ENUMv (GClosure *closure,
                                            GValue   *return_value,
                                            gpointer  instance,
                                            va_list   args,
                                            gpointer  marshal_data,
                                            int       n_params,
                                            GType    *param_types);

/* VOID:STRING (../gnome-builder/src/libide/core/ide-marshal.defs:13) */
#define ide_marshal_VOID__STRING	g_cclosure_marshal_VOID__STRING
#define ide_marshal_VOID__STRINGv	g_cclosure_marshal_VOID__STRINGv

/* VOID:STRING,STRING (../gnome-builder/src/libide/core/ide-marshal.defs:14) */
G_GNUC_INTERNAL
void ide_marshal_VOID__STRING_STRING (GClosure     *closure,
                                      GValue       *return_value,
                                      guint         n_param_values,
                                      const GValue *param_values,
                                      gpointer      invocation_hint,
                                      gpointer      marshal_data);
G_GNUC_INTERNAL
void ide_marshal_VOID__STRING_STRINGv (GClosure *closure,
                                       GValue   *return_value,
                                       gpointer  instance,
                                       va_list   args,
                                       gpointer  marshal_data,
                                       int       n_params,
                                       GType    *param_types);

/* VOID:STRING,VARIANT (../gnome-builder/src/libide/core/ide-marshal.defs:15) */
G_GNUC_INTERNAL
void ide_marshal_VOID__STRING_VARIANT (GClosure     *closure,
                                       GValue       *return_value,
                                       guint         n_param_values,
                                       const GValue *param_values,
                                       gpointer      invocation_hint,
                                       gpointer      marshal_data);
G_GNUC_INTERNAL
void ide_marshal_VOID__STRING_VARIANTv (GClosure *closure,
                                        GValue   *return_value,
                                        gpointer  instance,
                                        va_list   args,
                                        gpointer  marshal_data,
                                        int       n_params,
                                        GType    *param_types);

/* VOID:VOID (../gnome-builder/src/libide/core/ide-marshal.defs:16) */
#define ide_marshal_VOID__VOID	g_cclosure_marshal_VOID__VOID
#define ide_marshal_VOID__VOIDv	g_cclosure_marshal_VOID__VOIDv


G_END_DECLS
