/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __IPC_GIT_SERVICE_H__
#define __IPC_GIT_SERVICE_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.gnome.Builder.Git.Service */

#define IPC_TYPE_GIT_SERVICE (ipc_git_service_get_type ())
#define IPC_GIT_SERVICE(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), IPC_TYPE_GIT_SERVICE, IpcGitService))
#define IPC_IS_GIT_SERVICE(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), IPC_TYPE_GIT_SERVICE))
#define IPC_GIT_SERVICE_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), IPC_TYPE_GIT_SERVICE, IpcGitServiceIface))

struct _IpcGitService;
typedef struct _IpcGitService IpcGitService;
typedef struct _IpcGitServiceIface IpcGitServiceIface;

struct _IpcGitServiceIface
{
  GTypeInterface parent_iface;

  gboolean (*handle_clone) (
    IpcGitService *object,
    GDBusMethodInvocation *invocation,
    GUnixFDList *fd_list,
    const gchar *arg_url,
    const gchar *arg_location,
    const gchar *arg_branch,
    GVariant *arg_config_options,
    const gchar *arg_progress,
    GVariant *arg_pty_fd);

  gboolean (*handle_create) (
    IpcGitService *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_location,
    gboolean arg_is_bare);

  gboolean (*handle_discover) (
    IpcGitService *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_location);

  gboolean (*handle_list_remote_refs_by_kind) (
    IpcGitService *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_url,
    guint arg_kind);

  gboolean (*handle_load_config) (
    IpcGitService *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_open) (
    IpcGitService *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_location);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (IpcGitService, g_object_unref)
#endif

GType ipc_git_service_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *ipc_git_service_interface_info (void);
guint ipc_git_service_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void ipc_git_service_complete_discover (
    IpcGitService *object,
    GDBusMethodInvocation *invocation,
    const gchar *git_location);

void ipc_git_service_complete_open (
    IpcGitService *object,
    GDBusMethodInvocation *invocation,
    const gchar *repository);

void ipc_git_service_complete_create (
    IpcGitService *object,
    GDBusMethodInvocation *invocation,
    const gchar *git_location);

void ipc_git_service_complete_clone (
    IpcGitService *object,
    GDBusMethodInvocation *invocation,
    GUnixFDList *fd_list,
    const gchar *git_location);

void ipc_git_service_complete_load_config (
    IpcGitService *object,
    GDBusMethodInvocation *invocation,
    const gchar *config_path);

void ipc_git_service_complete_list_remote_refs_by_kind (
    IpcGitService *object,
    GDBusMethodInvocation *invocation,
    const gchar *const *refs);



/* D-Bus method calls: */
void ipc_git_service_call_discover (
    IpcGitService *proxy,
    const gchar *arg_location,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean ipc_git_service_call_discover_finish (
    IpcGitService *proxy,
    gchar ** out_git_location,
    GAsyncResult *res,
    GError **error);

gboolean ipc_git_service_call_discover_sync (
    IpcGitService *proxy,
    const gchar *arg_location,
    gchar ** out_git_location,
    GCancellable *cancellable,
    GError **error);

void ipc_git_service_call_open (
    IpcGitService *proxy,
    const gchar *arg_location,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean ipc_git_service_call_open_finish (
    IpcGitService *proxy,
    gchar ** out_repository,
    GAsyncResult *res,
    GError **error);

gboolean ipc_git_service_call_open_sync (
    IpcGitService *proxy,
    const gchar *arg_location,
    gchar ** out_repository,
    GCancellable *cancellable,
    GError **error);

void ipc_git_service_call_create (
    IpcGitService *proxy,
    const gchar *arg_location,
    gboolean arg_is_bare,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean ipc_git_service_call_create_finish (
    IpcGitService *proxy,
    gchar ** out_git_location,
    GAsyncResult *res,
    GError **error);

gboolean ipc_git_service_call_create_sync (
    IpcGitService *proxy,
    const gchar *arg_location,
    gboolean arg_is_bare,
    gchar ** out_git_location,
    GCancellable *cancellable,
    GError **error);

void ipc_git_service_call_clone (
    IpcGitService *proxy,
    const gchar *arg_url,
    const gchar *arg_location,
    const gchar *arg_branch,
    GVariant *arg_config_options,
    const gchar *arg_progress,
    GVariant *arg_pty_fd,
    GUnixFDList *fd_list,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean ipc_git_service_call_clone_finish (
    IpcGitService *proxy,
    gchar ** out_git_location,
    GUnixFDList **out_fd_list,
    GAsyncResult *res,
    GError **error);

gboolean ipc_git_service_call_clone_sync (
    IpcGitService *proxy,
    const gchar *arg_url,
    const gchar *arg_location,
    const gchar *arg_branch,
    GVariant *arg_config_options,
    const gchar *arg_progress,
    GVariant *arg_pty_fd,
    GUnixFDList  *fd_list,
    gchar ** out_git_location,
    GUnixFDList **out_fd_list,
    GCancellable *cancellable,
    GError **error);

void ipc_git_service_call_load_config (
    IpcGitService *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean ipc_git_service_call_load_config_finish (
    IpcGitService *proxy,
    gchar ** out_config_path,
    GAsyncResult *res,
    GError **error);

gboolean ipc_git_service_call_load_config_sync (
    IpcGitService *proxy,
    gchar ** out_config_path,
    GCancellable *cancellable,
    GError **error);

void ipc_git_service_call_list_remote_refs_by_kind (
    IpcGitService *proxy,
    const gchar *arg_url,
    guint arg_kind,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean ipc_git_service_call_list_remote_refs_by_kind_finish (
    IpcGitService *proxy,
    gchar *** out_refs,
    GAsyncResult *res,
    GError **error);

gboolean ipc_git_service_call_list_remote_refs_by_kind_sync (
    IpcGitService *proxy,
    const gchar *arg_url,
    guint arg_kind,
    gchar *** out_refs,
    GCancellable *cancellable,
    GError **error);



/* ---- */

#define IPC_TYPE_GIT_SERVICE_PROXY (ipc_git_service_proxy_get_type ())
#define IPC_GIT_SERVICE_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), IPC_TYPE_GIT_SERVICE_PROXY, IpcGitServiceProxy))
#define IPC_GIT_SERVICE_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), IPC_TYPE_GIT_SERVICE_PROXY, IpcGitServiceProxyClass))
#define IPC_GIT_SERVICE_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), IPC_TYPE_GIT_SERVICE_PROXY, IpcGitServiceProxyClass))
#define IPC_IS_GIT_SERVICE_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), IPC_TYPE_GIT_SERVICE_PROXY))
#define IPC_IS_GIT_SERVICE_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), IPC_TYPE_GIT_SERVICE_PROXY))

typedef struct _IpcGitServiceProxy IpcGitServiceProxy;
typedef struct _IpcGitServiceProxyClass IpcGitServiceProxyClass;
typedef struct _IpcGitServiceProxyPrivate IpcGitServiceProxyPrivate;

struct _IpcGitServiceProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  IpcGitServiceProxyPrivate *priv;
};

struct _IpcGitServiceProxyClass
{
  GDBusProxyClass parent_class;
};

GType ipc_git_service_proxy_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (IpcGitServiceProxy, g_object_unref)
#endif

void ipc_git_service_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
IpcGitService *ipc_git_service_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
IpcGitService *ipc_git_service_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void ipc_git_service_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
IpcGitService *ipc_git_service_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
IpcGitService *ipc_git_service_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define IPC_TYPE_GIT_SERVICE_SKELETON (ipc_git_service_skeleton_get_type ())
#define IPC_GIT_SERVICE_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), IPC_TYPE_GIT_SERVICE_SKELETON, IpcGitServiceSkeleton))
#define IPC_GIT_SERVICE_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), IPC_TYPE_GIT_SERVICE_SKELETON, IpcGitServiceSkeletonClass))
#define IPC_GIT_SERVICE_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), IPC_TYPE_GIT_SERVICE_SKELETON, IpcGitServiceSkeletonClass))
#define IPC_IS_GIT_SERVICE_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), IPC_TYPE_GIT_SERVICE_SKELETON))
#define IPC_IS_GIT_SERVICE_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), IPC_TYPE_GIT_SERVICE_SKELETON))

typedef struct _IpcGitServiceSkeleton IpcGitServiceSkeleton;
typedef struct _IpcGitServiceSkeletonClass IpcGitServiceSkeletonClass;
typedef struct _IpcGitServiceSkeletonPrivate IpcGitServiceSkeletonPrivate;

struct _IpcGitServiceSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  IpcGitServiceSkeletonPrivate *priv;
};

struct _IpcGitServiceSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType ipc_git_service_skeleton_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (IpcGitServiceSkeleton, g_object_unref)
#endif

IpcGitService *ipc_git_service_skeleton_new (void);


G_END_DECLS

#endif /* __IPC_GIT_SERVICE_H__ */
