
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#include "config.h"
#include "ide-gtk-enums.h"
#include "ide-animation.h"
#include "ide-scrubber-revealer.h"

#define C_ENUM(v) ((gint) v)
#define C_FLAGS(v) ((guint) v)

/* enumerations from "ide-animation.h" */

GType
ide_animation_mode_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (IDE_ANIMATION_LINEAR), "IDE_ANIMATION_LINEAR", "linear" },
        { C_ENUM (IDE_ANIMATION_EASE_IN_QUAD), "IDE_ANIMATION_EASE_IN_QUAD", "ease-in-quad" },
        { C_ENUM (IDE_ANIMATION_EASE_OUT_QUAD), "IDE_ANIMATION_EASE_OUT_QUAD", "ease-out-quad" },
        { C_ENUM (IDE_ANIMATION_EASE_IN_OUT_QUAD), "IDE_ANIMATION_EASE_IN_OUT_QUAD", "ease-in-out-quad" },
        { C_ENUM (IDE_ANIMATION_EASE_IN_CUBIC), "IDE_ANIMATION_EASE_IN_CUBIC", "ease-in-cubic" },
        { C_ENUM (IDE_ANIMATION_EASE_OUT_CUBIC), "IDE_ANIMATION_EASE_OUT_CUBIC", "ease-out-cubic" },
        { C_ENUM (IDE_ANIMATION_EASE_IN_OUT_CUBIC), "IDE_ANIMATION_EASE_IN_OUT_CUBIC", "ease-in-out-cubic" },
        { C_ENUM (IDE_ANIMATION_LAST), "IDE_ANIMATION_LAST", "last" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("IdeAnimationMode"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "ide-scrubber-revealer.h" */

GType
ide_scrubber_reveal_policy_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (IDE_SCRUBBER_REVEAL_POLICY_NEVER), "IDE_SCRUBBER_REVEAL_POLICY_NEVER", "never" },
        { C_ENUM (IDE_SCRUBBER_REVEAL_POLICY_AUTO), "IDE_SCRUBBER_REVEAL_POLICY_AUTO", "auto" },
        { C_ENUM (IDE_SCRUBBER_REVEAL_POLICY_ALWAYS), "IDE_SCRUBBER_REVEAL_POLICY_ALWAYS", "always" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("IdeScrubberRevealPolicy"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* Generated data ends here */

