/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __IPC_FLATPAK_SERVICE_H__
#define __IPC_FLATPAK_SERVICE_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.gnome.Builder.Flatpak.Service */

#define IPC_TYPE_FLATPAK_SERVICE (ipc_flatpak_service_get_type ())
#define IPC_FLATPAK_SERVICE(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), IPC_TYPE_FLATPAK_SERVICE, IpcFlatpakService))
#define IPC_IS_FLATPAK_SERVICE(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), IPC_TYPE_FLATPAK_SERVICE))
#define IPC_FLATPAK_SERVICE_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), IPC_TYPE_FLATPAK_SERVICE, IpcFlatpakServiceIface))

struct _IpcFlatpakService;
typedef struct _IpcFlatpakService IpcFlatpakService;
typedef struct _IpcFlatpakServiceIface IpcFlatpakServiceIface;

struct _IpcFlatpakServiceIface
{
  GTypeInterface parent_iface;



  gboolean (*handle_add_installation) (
    IpcFlatpakService *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_path,
    gboolean arg_is_user);

  gboolean (*handle_get_runtime) (
    IpcFlatpakService *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_id);

  gboolean (*handle_install) (
    IpcFlatpakService *object,
    GDBusMethodInvocation *invocation,
    const gchar *const *arg_full_ref_name,
    gboolean arg_if_not_exists,
    const gchar *arg_transfer,
    const gchar *arg_parent_window);

  gboolean (*handle_list_runtimes) (
    IpcFlatpakService *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_resolve_extension) (
    IpcFlatpakService *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_sdk,
    const gchar *arg_extension);

  gboolean (*handle_runtime_is_known) (
    IpcFlatpakService *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_name);

  const gchar * (*get_config_dir) (IpcFlatpakService *object);

  const gchar * (*get_default_arch) (IpcFlatpakService *object);

  void (*runtime_added) (
    IpcFlatpakService *object,
    GVariant *arg_runtime);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (IpcFlatpakService, g_object_unref)
#endif

GType ipc_flatpak_service_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *ipc_flatpak_service_interface_info (void);
guint ipc_flatpak_service_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void ipc_flatpak_service_complete_add_installation (
    IpcFlatpakService *object,
    GDBusMethodInvocation *invocation);

void ipc_flatpak_service_complete_list_runtimes (
    IpcFlatpakService *object,
    GDBusMethodInvocation *invocation,
    GVariant *runtimes);

void ipc_flatpak_service_complete_get_runtime (
    IpcFlatpakService *object,
    GDBusMethodInvocation *invocation,
    GVariant *runtime);

void ipc_flatpak_service_complete_runtime_is_known (
    IpcFlatpakService *object,
    GDBusMethodInvocation *invocation,
    gboolean is_known,
    gint64 download_size);

void ipc_flatpak_service_complete_install (
    IpcFlatpakService *object,
    GDBusMethodInvocation *invocation);

void ipc_flatpak_service_complete_resolve_extension (
    IpcFlatpakService *object,
    GDBusMethodInvocation *invocation,
    const gchar *resolved);



/* D-Bus signal emissions functions: */
void ipc_flatpak_service_emit_runtime_added (
    IpcFlatpakService *object,
    GVariant *arg_runtime);



/* D-Bus method calls: */
void ipc_flatpak_service_call_add_installation (
    IpcFlatpakService *proxy,
    const gchar *arg_path,
    gboolean arg_is_user,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean ipc_flatpak_service_call_add_installation_finish (
    IpcFlatpakService *proxy,
    GAsyncResult *res,
    GError **error);

gboolean ipc_flatpak_service_call_add_installation_sync (
    IpcFlatpakService *proxy,
    const gchar *arg_path,
    gboolean arg_is_user,
    GCancellable *cancellable,
    GError **error);

void ipc_flatpak_service_call_list_runtimes (
    IpcFlatpakService *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean ipc_flatpak_service_call_list_runtimes_finish (
    IpcFlatpakService *proxy,
    GVariant ** out_runtimes,
    GAsyncResult *res,
    GError **error);

gboolean ipc_flatpak_service_call_list_runtimes_sync (
    IpcFlatpakService *proxy,
    GVariant ** out_runtimes,
    GCancellable *cancellable,
    GError **error);

void ipc_flatpak_service_call_get_runtime (
    IpcFlatpakService *proxy,
    const gchar *arg_id,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean ipc_flatpak_service_call_get_runtime_finish (
    IpcFlatpakService *proxy,
    GVariant ** out_runtime,
    GAsyncResult *res,
    GError **error);

gboolean ipc_flatpak_service_call_get_runtime_sync (
    IpcFlatpakService *proxy,
    const gchar *arg_id,
    GVariant ** out_runtime,
    GCancellable *cancellable,
    GError **error);

void ipc_flatpak_service_call_runtime_is_known (
    IpcFlatpakService *proxy,
    const gchar *arg_name,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean ipc_flatpak_service_call_runtime_is_known_finish (
    IpcFlatpakService *proxy,
    gboolean* out_is_known,
    gint64* out_download_size,
    GAsyncResult *res,
    GError **error);

gboolean ipc_flatpak_service_call_runtime_is_known_sync (
    IpcFlatpakService *proxy,
    const gchar *arg_name,
    gboolean* out_is_known,
    gint64* out_download_size,
    GCancellable *cancellable,
    GError **error);

void ipc_flatpak_service_call_install (
    IpcFlatpakService *proxy,
    const gchar *const *arg_full_ref_name,
    gboolean arg_if_not_exists,
    const gchar *arg_transfer,
    const gchar *arg_parent_window,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean ipc_flatpak_service_call_install_finish (
    IpcFlatpakService *proxy,
    GAsyncResult *res,
    GError **error);

gboolean ipc_flatpak_service_call_install_sync (
    IpcFlatpakService *proxy,
    const gchar *const *arg_full_ref_name,
    gboolean arg_if_not_exists,
    const gchar *arg_transfer,
    const gchar *arg_parent_window,
    GCancellable *cancellable,
    GError **error);

void ipc_flatpak_service_call_resolve_extension (
    IpcFlatpakService *proxy,
    const gchar *arg_sdk,
    const gchar *arg_extension,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean ipc_flatpak_service_call_resolve_extension_finish (
    IpcFlatpakService *proxy,
    gchar ** out_resolved,
    GAsyncResult *res,
    GError **error);

gboolean ipc_flatpak_service_call_resolve_extension_sync (
    IpcFlatpakService *proxy,
    const gchar *arg_sdk,
    const gchar *arg_extension,
    gchar ** out_resolved,
    GCancellable *cancellable,
    GError **error);



/* D-Bus property accessors: */
const gchar *ipc_flatpak_service_get_config_dir (IpcFlatpakService *object);
gchar *ipc_flatpak_service_dup_config_dir (IpcFlatpakService *object);
void ipc_flatpak_service_set_config_dir (IpcFlatpakService *object, const gchar *value);

const gchar *ipc_flatpak_service_get_default_arch (IpcFlatpakService *object);
gchar *ipc_flatpak_service_dup_default_arch (IpcFlatpakService *object);
void ipc_flatpak_service_set_default_arch (IpcFlatpakService *object, const gchar *value);


/* ---- */

#define IPC_TYPE_FLATPAK_SERVICE_PROXY (ipc_flatpak_service_proxy_get_type ())
#define IPC_FLATPAK_SERVICE_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), IPC_TYPE_FLATPAK_SERVICE_PROXY, IpcFlatpakServiceProxy))
#define IPC_FLATPAK_SERVICE_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), IPC_TYPE_FLATPAK_SERVICE_PROXY, IpcFlatpakServiceProxyClass))
#define IPC_FLATPAK_SERVICE_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), IPC_TYPE_FLATPAK_SERVICE_PROXY, IpcFlatpakServiceProxyClass))
#define IPC_IS_FLATPAK_SERVICE_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), IPC_TYPE_FLATPAK_SERVICE_PROXY))
#define IPC_IS_FLATPAK_SERVICE_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), IPC_TYPE_FLATPAK_SERVICE_PROXY))

typedef struct _IpcFlatpakServiceProxy IpcFlatpakServiceProxy;
typedef struct _IpcFlatpakServiceProxyClass IpcFlatpakServiceProxyClass;
typedef struct _IpcFlatpakServiceProxyPrivate IpcFlatpakServiceProxyPrivate;

struct _IpcFlatpakServiceProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  IpcFlatpakServiceProxyPrivate *priv;
};

struct _IpcFlatpakServiceProxyClass
{
  GDBusProxyClass parent_class;
};

GType ipc_flatpak_service_proxy_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (IpcFlatpakServiceProxy, g_object_unref)
#endif

void ipc_flatpak_service_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
IpcFlatpakService *ipc_flatpak_service_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
IpcFlatpakService *ipc_flatpak_service_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void ipc_flatpak_service_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
IpcFlatpakService *ipc_flatpak_service_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
IpcFlatpakService *ipc_flatpak_service_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define IPC_TYPE_FLATPAK_SERVICE_SKELETON (ipc_flatpak_service_skeleton_get_type ())
#define IPC_FLATPAK_SERVICE_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), IPC_TYPE_FLATPAK_SERVICE_SKELETON, IpcFlatpakServiceSkeleton))
#define IPC_FLATPAK_SERVICE_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), IPC_TYPE_FLATPAK_SERVICE_SKELETON, IpcFlatpakServiceSkeletonClass))
#define IPC_FLATPAK_SERVICE_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), IPC_TYPE_FLATPAK_SERVICE_SKELETON, IpcFlatpakServiceSkeletonClass))
#define IPC_IS_FLATPAK_SERVICE_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), IPC_TYPE_FLATPAK_SERVICE_SKELETON))
#define IPC_IS_FLATPAK_SERVICE_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), IPC_TYPE_FLATPAK_SERVICE_SKELETON))

typedef struct _IpcFlatpakServiceSkeleton IpcFlatpakServiceSkeleton;
typedef struct _IpcFlatpakServiceSkeletonClass IpcFlatpakServiceSkeletonClass;
typedef struct _IpcFlatpakServiceSkeletonPrivate IpcFlatpakServiceSkeletonPrivate;

struct _IpcFlatpakServiceSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  IpcFlatpakServiceSkeletonPrivate *priv;
};

struct _IpcFlatpakServiceSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType ipc_flatpak_service_skeleton_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (IpcFlatpakServiceSkeleton, g_object_unref)
#endif

IpcFlatpakService *ipc_flatpak_service_skeleton_new (void);


G_END_DECLS

#endif /* __IPC_FLATPAK_SERVICE_H__ */
