/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __IPC_FLATPAK_TRANSFER_H__
#define __IPC_FLATPAK_TRANSFER_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.gnome.Builder.Flatpak.Transfer */

#define IPC_TYPE_FLATPAK_TRANSFER (ipc_flatpak_transfer_get_type ())
#define IPC_FLATPAK_TRANSFER(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), IPC_TYPE_FLATPAK_TRANSFER, IpcFlatpakTransfer))
#define IPC_IS_FLATPAK_TRANSFER(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), IPC_TYPE_FLATPAK_TRANSFER))
#define IPC_FLATPAK_TRANSFER_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), IPC_TYPE_FLATPAK_TRANSFER, IpcFlatpakTransferIface))

struct _IpcFlatpakTransfer;
typedef struct _IpcFlatpakTransfer IpcFlatpakTransfer;
typedef struct _IpcFlatpakTransferIface IpcFlatpakTransferIface;

struct _IpcFlatpakTransferIface
{
  GTypeInterface parent_iface;



  gboolean (*handle_confirm) (
    IpcFlatpakTransfer *object,
    GDBusMethodInvocation *invocation,
    const gchar *const *arg_refs);

  gdouble  (*get_fraction) (IpcFlatpakTransfer *object);

  const gchar * (*get_message) (IpcFlatpakTransfer *object);

  void (*cancel) (
    IpcFlatpakTransfer *object);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (IpcFlatpakTransfer, g_object_unref)
#endif

GType ipc_flatpak_transfer_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *ipc_flatpak_transfer_interface_info (void);
guint ipc_flatpak_transfer_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void ipc_flatpak_transfer_complete_confirm (
    IpcFlatpakTransfer *object,
    GDBusMethodInvocation *invocation);



/* D-Bus signal emissions functions: */
void ipc_flatpak_transfer_emit_cancel (
    IpcFlatpakTransfer *object);



/* D-Bus method calls: */
void ipc_flatpak_transfer_call_confirm (
    IpcFlatpakTransfer *proxy,
    const gchar *const *arg_refs,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean ipc_flatpak_transfer_call_confirm_finish (
    IpcFlatpakTransfer *proxy,
    GAsyncResult *res,
    GError **error);

gboolean ipc_flatpak_transfer_call_confirm_sync (
    IpcFlatpakTransfer *proxy,
    const gchar *const *arg_refs,
    GCancellable *cancellable,
    GError **error);



/* D-Bus property accessors: */
gdouble ipc_flatpak_transfer_get_fraction (IpcFlatpakTransfer *object);
void ipc_flatpak_transfer_set_fraction (IpcFlatpakTransfer *object, gdouble value);

const gchar *ipc_flatpak_transfer_get_message (IpcFlatpakTransfer *object);
gchar *ipc_flatpak_transfer_dup_message (IpcFlatpakTransfer *object);
void ipc_flatpak_transfer_set_message (IpcFlatpakTransfer *object, const gchar *value);


/* ---- */

#define IPC_TYPE_FLATPAK_TRANSFER_PROXY (ipc_flatpak_transfer_proxy_get_type ())
#define IPC_FLATPAK_TRANSFER_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), IPC_TYPE_FLATPAK_TRANSFER_PROXY, IpcFlatpakTransferProxy))
#define IPC_FLATPAK_TRANSFER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), IPC_TYPE_FLATPAK_TRANSFER_PROXY, IpcFlatpakTransferProxyClass))
#define IPC_FLATPAK_TRANSFER_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), IPC_TYPE_FLATPAK_TRANSFER_PROXY, IpcFlatpakTransferProxyClass))
#define IPC_IS_FLATPAK_TRANSFER_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), IPC_TYPE_FLATPAK_TRANSFER_PROXY))
#define IPC_IS_FLATPAK_TRANSFER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), IPC_TYPE_FLATPAK_TRANSFER_PROXY))

typedef struct _IpcFlatpakTransferProxy IpcFlatpakTransferProxy;
typedef struct _IpcFlatpakTransferProxyClass IpcFlatpakTransferProxyClass;
typedef struct _IpcFlatpakTransferProxyPrivate IpcFlatpakTransferProxyPrivate;

struct _IpcFlatpakTransferProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  IpcFlatpakTransferProxyPrivate *priv;
};

struct _IpcFlatpakTransferProxyClass
{
  GDBusProxyClass parent_class;
};

GType ipc_flatpak_transfer_proxy_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (IpcFlatpakTransferProxy, g_object_unref)
#endif

void ipc_flatpak_transfer_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
IpcFlatpakTransfer *ipc_flatpak_transfer_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
IpcFlatpakTransfer *ipc_flatpak_transfer_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void ipc_flatpak_transfer_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
IpcFlatpakTransfer *ipc_flatpak_transfer_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
IpcFlatpakTransfer *ipc_flatpak_transfer_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define IPC_TYPE_FLATPAK_TRANSFER_SKELETON (ipc_flatpak_transfer_skeleton_get_type ())
#define IPC_FLATPAK_TRANSFER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), IPC_TYPE_FLATPAK_TRANSFER_SKELETON, IpcFlatpakTransferSkeleton))
#define IPC_FLATPAK_TRANSFER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), IPC_TYPE_FLATPAK_TRANSFER_SKELETON, IpcFlatpakTransferSkeletonClass))
#define IPC_FLATPAK_TRANSFER_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), IPC_TYPE_FLATPAK_TRANSFER_SKELETON, IpcFlatpakTransferSkeletonClass))
#define IPC_IS_FLATPAK_TRANSFER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), IPC_TYPE_FLATPAK_TRANSFER_SKELETON))
#define IPC_IS_FLATPAK_TRANSFER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), IPC_TYPE_FLATPAK_TRANSFER_SKELETON))

typedef struct _IpcFlatpakTransferSkeleton IpcFlatpakTransferSkeleton;
typedef struct _IpcFlatpakTransferSkeletonClass IpcFlatpakTransferSkeletonClass;
typedef struct _IpcFlatpakTransferSkeletonPrivate IpcFlatpakTransferSkeletonPrivate;

struct _IpcFlatpakTransferSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  IpcFlatpakTransferSkeletonPrivate *priv;
};

struct _IpcFlatpakTransferSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType ipc_flatpak_transfer_skeleton_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (IpcFlatpakTransferSkeleton, g_object_unref)
#endif

IpcFlatpakTransfer *ipc_flatpak_transfer_skeleton_new (void);


G_END_DECLS

#endif /* __IPC_FLATPAK_TRANSFER_H__ */
